/**
 *
 * Magnolia SimpleMedia Module (http://www.openmindlab.com/lab/products/media.html)
 * Copyright (C)2008 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package net.sourceforge.openutils.mgnlmedia.media.dialog;

import freemarker.template.TemplateException;
import freemarker.template.TemplateMethodModel;
import freemarker.template.TemplateModelException;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.core.SystemProperty;
import info.magnolia.cms.gui.control.ControlImpl;
import info.magnolia.cms.gui.control.FreemarkerControl;
import info.magnolia.cms.i18n.Messages;
import info.magnolia.cms.i18n.MessagesUtil;
import info.magnolia.cms.security.AccessDeniedException;
import info.magnolia.cms.util.NodeDataUtil;
import info.magnolia.context.MgnlContext;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaConfigurationManager;
import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaTypeConfiguration;
import net.sourceforge.openutils.mgnlmedia.media.lifecycle.MediaModule;

import org.apache.commons.lang.exception.NestableRuntimeException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * This dialogs draws the control that allows to select a media from repository and store its uuid
 * @author molaschi
 * @version $Id: DialogSelectMedia.java 1576 2009-12-31 15:29:37Z fgiust $
 */
@SuppressWarnings("unchecked")
public class DialogSelectMedia extends DialogCustomSaveHandler
{

    /**
     * Logger.
     */
    private Logger log = LoggerFactory.getLogger(DialogSelectMedia.class);

    private Content media;

    private Map configuration;

    /**
     * {@inheritDoc}
     */
    @Override
    protected List readValues()
    {

        List values = new ArrayList();

        if (this.getStorageNode() != null)
        {
            try
            {
                // cycles on website content node to get multiple value
                int size = this.getStorageNode().getContent(this.getName()).getNodeDataCollection().size();
                for (int i = 0; i < size; i++)
                {
                    NodeData data = this.getStorageNode().getContent(this.getName()).getNodeData("" + i);
                    values.add(data.getString());
                }
            }
            catch (PathNotFoundException e)
            {
                // not yet existing: OK
            }
            catch (RepositoryException re)
            {
                log.error("can't set values", re);
            }
        }
        return values;
    }

    /**
     * Get a recursive map view of a content node
     * @param node content node
     * @return recursive map view on content node properties and children
     * @throws RepositoryException repository exception
     * @throws AccessDeniedException access denied
     */
    protected Map getSubNodes(Content node) throws RepositoryException, AccessDeniedException
    {

        Map values = new LinkedHashMap();

        // cycles on properties and stores them in map
        Collection properties = node.getNodeDataCollection();

        if (properties != null && properties.size() > 0)
        {
            Iterator propertiesIt = properties.iterator();
            while (propertiesIt.hasNext())
            {
                NodeData property = (NodeData) propertiesIt.next();
                values.put(property.getName(), NodeDataUtil.getValueObject(property));
            }
        }

        // cycle on children
        Collection children = node.getChildren(ItemType.CONTENTNODE);
        if (children != null && children.size() > 0)
        {
            Iterator childrenIt = children.iterator();
            while (childrenIt.hasNext())
            {
                Content child = (Content) childrenIt.next();

                // gets sub map
                Map subValues = getSubNodes(child);
                // stores it in map
                values.put(child.getName(), subValues);
            }
        }

        return values;
    }

    /**
     * {@inheritDoc}
     */

    @Override
    public void init(HttpServletRequest request, HttpServletResponse response, Content websiteNode, Content configNode)
        throws RepositoryException
    {
        super.init(request, response, websiteNode, configNode);
        configuration = this.getSubNodes(configNode);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void drawHtml(Writer out) throws IOException
    {

        Map parameters = new HashMap();
        parameters.put("name", this.getName());
        parameters.put("value", this.getValue());
        parameters.put("values", this.getValues());
        parameters.put("request", this.getRequest());
        parameters.put("debug", SystemProperty.getBooleanProperty("magnolia.develop"));
        parameters.put("configuration", this.configuration);
        parameters.put("thumbnailUrl", this.getThumbnailUrl());
        parameters.put("filename", this.getFilename());
        parameters.put("reqGet", new RequestGetAttribute());
        parameters.put("reqSet", new RequestSetAttribute());
        parameters.put("msgs", this.getMessages());
        this.drawHtmlPre(out);

        try
        {
            FreemarkerControl control = new FreemarkerControl("multiple".equals(this.getConfigValue("valueType"))
                ? ControlImpl.VALUETYPE_MULTIPLE
                : ControlImpl.VALUETYPE_SINGLE);
            control.setType(this.getConfigValue("type", PropertyType.TYPENAME_STRING));
            control.setName(this.getName());
            control.drawHtml(out, getPath(), parameters);
        }

        catch (TemplateException ex)
        {
            log.error("Error processing dialog template:", ex);
            throw new NestableRuntimeException(ex);
        }

        this.drawHtmlPost(out);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Messages getMessages()
    {
        return MessagesUtil.chain("net.sourceforge.openutils.mgnlmedia.media.lang.messages", super.getMessages());
    }

    protected String getPath()
    {
        return "dialog/selectMedia.ftl";
    }

    /**
     * return current media content
     * @return media content
     * @throws RepositoryException exception retrieving media
     */
    protected Content getMedia() throws RepositoryException
    {
        if (this.media == null && this.getValue() != null && this.getValue().length() > 0)
        {
            HierarchyManager hm = MgnlContext.getHierarchyManager(MediaModule.REPO);
            this.media = hm.getContentByUUID(this.getValue());
        }
        return this.media;
    }

    /**
     * get thumbnail url
     * @return thumbnail url
     */
    public String getThumbnailUrl()
    {
        Content media;

        try
        {
            media = getMedia();
        }
        catch (RepositoryException ex)
        {
            log.error("Error retriving media {}", value, ex);
            return null;
        }

        if (media != null)
        {
            MediaTypeConfiguration mtc = MediaConfigurationManager.getInstance().getMediaTypeConfigurationFromMedia(
                media);
            return this.getRequest().getContextPath() + mtc.getHandler().getThumbnailUrl(media);
        }
        else
        {
            return null;
        }
    }

    /**
     * Filename
     * @return filename
     */
    public String getFilename()
    {
        Content media;

        try
        {
            media = getMedia();
        }
        catch (RepositoryException ex)
        {
            log.error("Error retriving media {}", value, ex);
            return null;
        }

        if (media != null)
        {
            MediaTypeConfiguration mtc = MediaConfigurationManager.getInstance().getMediaTypeConfigurationFromMedia(
                media);
            return mtc.getHandler().getFilename(media);
        }
        else
        {
            return null;
        }
    }

    /**
     * Freemarker method for setting request attributes
     * @author molaschi
     * @version $Id: DialogSelectMedia.java 1576 2009-12-31 15:29:37Z fgiust $
     */
    public class RequestSetAttribute implements TemplateMethodModel
    {

        /**
         * {@inheritDoc}
         */
        public Object exec(List arguments) throws TemplateModelException
        {
            if (arguments != null && arguments.size() == 3 && arguments.get(0) instanceof HttpServletRequest)
            {
                HttpServletRequest req = (HttpServletRequest) arguments.get(0);
                String name = (String) arguments.get(1);
                Object value = arguments.get(2);
                req.setAttribute(name, value);
            }

            return TemplateMethodModel.NOTHING;
        }

    }

    /**
     * Freemarker method for getting request attributes
     * @author molaschi
     * @version $Id: DialogSelectMedia.java 1576 2009-12-31 15:29:37Z fgiust $
     */
    public class RequestGetAttribute implements TemplateMethodModel
    {

        /**
         * {@inheritDoc}
         */
        public Object exec(List arguments) throws TemplateModelException
        {
            if (arguments != null && arguments.size() == 2 && arguments.get(0) instanceof HttpServletRequest)
            {
                HttpServletRequest req = (HttpServletRequest) arguments.get(0);
                String name = (String) arguments.get(1);
                return req.getAttribute(name);
            }

            return null;
        }

    }

}
